@@ -480,8 +480,14 @@ void options_print_usage()
 
 void options_print_version()
 {
-  printf("%s\n", VERSION_STRING_0);
-  printf("%s\n", VERSION_STRING_1);
+  printf("%s\n", VERSION_STRING);
+#if defined(__clang__)
+  printf("built using CLANG %s with %s crypto library.\n", __clang_version__, CRYPTO_LIB_NAME);
+#elif defined(__GNUC__)
+  printf("built using GCC %d.%d.%d with %s crypto library\n", __GNUC__, __GNUC_MINOR__, __GNUC_PATCHLEVEL__, CRYPTO_LIB_NAME);
+#else
+  printf("built using an unknown compiler with %s crypto library\n", CRYPTO_LIB_NAME, CRYPTO_LIB_NAME);
+#endif
 }
 
 void options_print(options_t* opt)
